/* Name: hardware.h
 */

/*
General Description:
This module defines hardware properties and configuration choices.
*/

#pragma once

/* configuration options: */
#define USE_DCD_REPORTING       1
/* If this option is defined to 1, the driver will report carrier detect when
 * the serial device is opened. This is useful on some Unix platforms to allow
 * using the /dev/tty* instead of /dev/cu* devices.
 * Setting this option to 0 saves a couple of bytes in flash and RAM memory.
 */
#define ENABLE_DEBUG_INTERFACE  0
/* If this option is defined to 1, the device buffers serial data (read from
 * the the ISP connector) and makes it available through vendor specific
 * requests to host based software. This is useful to display debug information
 * sent by the target.
 * Setting this option to 0 saves a couple of bytes in flash memory.
 */
#define ENABLE_HID_INTERFACE    0
/* If this option is defined to 1, the device implements a custom HID type
 * interface to send and receive STK500 serial data. If both, the CDC-ACM and
 * HID interface are enabled, a jumper selects which one is used.
 */
#define ENABLE_CDC_INTERFACE    1
/* If this option is defined to 1, the device implements a CDC-ACM modem,
 * emulating the behavior of the STK500 board on a virtual COM port.
 */
#define ENABLE_HVPROG           1
/* If this option is defined to 1, the high voltage programmer is enabled.
 */
#define HW_CDC_PACKET_SIZE      8
/* Size of bulk transfer packets.
 */

/* Hardware, Portbelegung AT90USB8U2 und hher
PB0 14	!SS	DATA0
PB1 15	SCK	DATA1, ISP(3), ESP(3)
PB2 16	MOSI	DATA2, ISP(4), ESP(4)
PB3 17	MISO	DATA3, ISP(1), ESP(1)
PB4 18	T1	DATA4
PB5 19		DATA5
PB6 20		DATA6
PB7 21	OC0A	DATA7

PC0  2	XTAL2	Quarz 16 MHz
PC1 24	!Reset	ISP(5)
PC2  5		R/B
PC3  -
PC4 26		Speisespannung Ucc fr Sockel und ESP(2)
PC5 25	OC1B	Upp: Programmierspannungs-Vorgabe 1/3, ESP(5)
PC6 23	OC1A	Wechselspannungsausgang fr 15-V-Kaskade
PC7 22	ICP	LED gegen 00

PD0  6	OC0B	XTAL1: Takt fr Target
PD1  7		!OE
PD2  8	RxD	!WR
PD3  9	TxD	BS1 (Bank-Auswahl)
PD4 10		XA0
PD5 11	XCK	XA1
PD6 12	!RTS	BS2 (Bank-Auswahl)
PD7 13	!CTS	PAGEL
*/

/* The following defines can be used with the PORT_* macros from utils.h */

#if USBASP_HARDWARE         /* USBasp hardware from www.fischl.de/usbasp/ */

#undef ENABLE_HVPROG
#define ENABLE_HVPROG       0
#define METABOARD_HARDWARE  1   /* most settings are the same as for metaboard */

#define HWPIN_LED           C,7

#define HWPIN_ISP_RESET     C,5
#define HWPIN_ISP_MOSI      B,2
#define HWPIN_ISP_MISO      B,3
#define HWPIN_ISP_SCK       B,1

#elif METABOARD_HARDWARE    /* Metaboard hardware from www.obdev.at/goto?t=metaboard */

#undef ENABLE_HVPROG
#define ENABLE_HVPROG       0

#define HWPIN_LED           C, 1
#define HWPIN_ISP_SUPPLY1   C, 3
#define HWPIN_ISP_SUPPLY2   C, 4

#define HWPIN_ISP_CLK       B, 1
#define HWPIN_ISP_RESET     B, 2
#define HWPIN_ISP_MOSI      B, 3
#define HWPIN_ISP_MISO      B, 4
#define HWPIN_ISP_SCK       B, 5

#define HWPIN_ISP_TXD       D, 0
#define HWPIN_ISP_RXD       D, 1

#else                       /* Standard AVR-Doper hardware */

#define HWPIN_HVSP_SUPPLY   B, 0
#define HWPIN_SMPS_OUT      B, 1
#define HWPIN_HVSP_HVRESET  B, 2
#define HWPIN_HVSP_SCI      B, 3
#define HWPIN_ISP_CLK       B, 3
#define HWPIN_ISP_DRIVER    B, 4
#define HWPIN_LED           B, 5

#define HWPIN_ADC_SMPS      C, 0
#define HWPIN_ADC_VTARGET   C, 1
#define HWPIN_ISP_SCK       C, 2
#define HWPIN_ISP_MISO      C, 3
#define HWPIN_ISP_MOSI      C, 4
#define HWPIN_ISP_RESET     C, 5
#define HWPIN_HVSP_RESET    C, 5

#define HWPIN_ISP_TXD       D, 0
#define HWPIN_ISP_RXD       D, 1
#define HWPIN_USB_DPLUS     D, 2
#define HWPIN_USB_DMINUS    D, 3
#define HWPIN_JUMPER        D, 4
#define HWPIN_HVSP_SII      D, 5
#define HWPIN_HVSP_SDI      D, 6
#define HWPIN_HVSP_SDO      D, 7

#endif

#include <avr/io.h>
