#include "hardware.h"
#include <avr/pgmspace.h>
#include <string.h>
#include "oddebug.h"
#include "stk500protocol.h"
#include "utils.h"
#include "isp.h"
#include "hvprog.h"
#include "timer.h"

/* The following versions are reported to the programming software: */
#define STK_VERSION_HW      1
#define STK_VERSION_MAJOR   2
#define STK_VERSION_MINOR   4

#define RX_TIMEOUT      200 /* timeout in milliseconds */

#define STK_TXMSG_START 5

namespace stk{

static word rxPos;
static Word<> rxLen;
static bool rxBlockAvailable;

byte rxBuffer[BUFFER_SIZE],txBuffer[BUFFER_SIZE];
static word txPos, txLen;

StkParam stkParam = {{
 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0,
 STK_VERSION_HW, STK_VERSION_MAJOR, STK_VERSION_MINOR, 0, 50, 0, 1, 0x80,
 2, 0, 0xaa, 0, 0, 0, 0, 0}};

Word<dword> Address;

/* ------------------------------------------------------------------------- */

void IncrementAddress() {Address++;}

/* ------------------------------------------------------------------------- */

static void SetTxMessage(word len) {
 byte*p = txBuffer;
 *p++ = STK_STX;
 *p++ = rxBuffer[1];  /* sequence number */
 *p++ = len>>8;
 *p++ = byte(len);
 *p++ = STK_TOKEN;
 txPos = 0;
 len += 6;	// Anzahl Sendebytes inklusive Prfbyte
 txLen = len--;
 p = txBuffer;
 byte sum=0;
 while(len--) sum ^= *p++;
 *p = sum;
}

/* ------------------------------------------------------------------------- */

static void setParameter() {
 switch (gParam.parameter.index) {
  case STK_PARAM_OSC_PSCALE:
//        HW_SET_T2_PRESCALER(gParam.parameter.value);
  break;
  case STK_PARAM_OSC_CMATCH:
//        OCR2 = gParam.parameter.value;
  break;
 }
 stkParam.bytes[gParam.parameter.index&0x1f] = gParam.parameter.value;
}

static uchar getParameter() {
 switch (gParam.parameter.index) {
  case STK_PARAM_OSC_PSCALE:;
//        return HW_GET_T2_PRESCALER();
  case STK_PARAM_OSC_CMATCH:;
//        return OCR2;
 }
 return stkParam.bytes[gParam.parameter.index&0x1f];
}

/* ------------------------------------------------------------------------- */

/* Use defines for the switch statement so that we can choose between an
 * if()else if() and a switch/case based implementation. switch() is more
 * efficient for a LARGE set of sequential choices, if() is better in all other
 * cases.
 */
#define SWITCH_START        switch(cmd){{
#define SWITCH_CASE(value)  }break; case (value):{
#define SWITCH_CASE2(v1,v2) }break; case (v1): case(v2):{
#define SWITCH_CASE3(v1,v2,v3) }break; case (v1): case(v2): case(v3):{
#define SWITCH_CASE4(v1,v2,v3,v4) }break; case (v1): case(v2): case(v3): case(v4):{
#define SWITCH_DEFAULT      }break; default:{
#define SWITCH_END          }}

void EvaluateRxMessage() { /* not static to prevent inlining */
 Word<> len;
 len = 2;  /* defaults to cmd + error code */
 byte cmd = gParam.maincmd;
 gResult.maincmd = cmd;
 gResult.status = STK_STATUS_CMD_OK;
 switch (cmd) {
  case STK_CMD_SIGN_ON:		strcpy_P(gResult.text,PSTR("\008STK500_2")); len[0] = 11; break;
  case STK_CMD_SET_PARAMETER:	setParameter(); break;
  case STK_CMD_GET_PARAMETER:	gResult.text[0] = getParameter(); len[0] = 3; break;
  case STK_CMD_OSCCAL:		gResult.status = STK_STATUS_CMD_FAILED; break;      /* not yet implemented */
  case STK_CMD_LOAD_ADDRESS:	Address = dwswap(gParam.beAddress); break;
  case STK_CMD_SET_CONTROL_STACK:
        /* AVR Studio sends:
        1b 08 00 21 0e 2d 
        4c 0c 1c 2c 3c 64 74 66
        68 78 68 68 7a 6a 68 78
        78 7d 6d 0c 80 40 20 10
        11 08 04 02 03 08 04 00
        bf
        */
        /* dummy: ignore */
  break;
  case STK_CMD_ENTER_PROGMODE_HVSP:	hv::spEnterProgmode(); break;
  case STK_CMD_LEAVE_PROGMODE_HVSP:	hv::spLeaveProgmode(); break;
  case STK_CMD_CHIP_ERASE_HVSP:		gResult.status = hv::spChipErase(); break;
  case STK_CMD_PROGRAM_FLASH_HVSP:	gResult.status = hv::ProgramMemory(false); break;
  case STK_CMD_READ_FLASH_HVSP:		len = 1 + hv::ReadMemory(false); break;
  case STK_CMD_PROGRAM_EEPROM_HVSP:	gResult.status = hv::ProgramMemory(true); break;
  case STK_CMD_READ_EEPROM_HVSP:	len = 1 + hv::ReadMemory(true);	break;
  case STK_CMD_PROGRAM_FUSE_HVSP:	gResult.status = hv::spProgramFuse(); break;
  case STK_CMD_READ_FUSE_HVSP:		gResult.data[0] = hv::ReadFuse(); len[0] = 3; break;
  case STK_CMD_PROGRAM_LOCK_HVSP:	gResult.status = hv::spProgramLock();	break;
  case STK_CMD_READ_LOCK_HVSP:		gResult.data[0] = hv::ReadLock(); len[0] = 3; break;
  case STK_CMD_READ_SIGNATURE_HVSP:	gResult.data[0] = hv::ReadSignature(); len[0] = 3; break;
  case STK_CMD_READ_OSCCAL_HVSP:	gResult.data[0] = hv::ReadOsccal(); len[0] = 3; break;

  case STK_CMD_ENTER_PROGMODE_PP:	hv::ppEnterProgmode(); break;
  case STK_CMD_LEAVE_PROGMODE_PP:	hv::ppLeaveProgmode(); break;
  case STK_CMD_CHIP_ERASE_PP:		gResult.status = hv::ppChipErase(); break;
  case STK_CMD_PROGRAM_FLASH_PP:	gResult.status = hv::ProgramMemory(false); break;
  case STK_CMD_READ_FLASH_PP:		len = 1 + hv::ReadMemory(false); break;
  case STK_CMD_PROGRAM_EEPROM_PP:	gResult.status = hv::ProgramMemory(true); break;
  case STK_CMD_READ_EEPROM_PP:		len = 1 + hv::ReadMemory(true); break;
  case STK_CMD_PROGRAM_FUSE_PP:		gResult.status = hv::ppProgramFuse(); break;
  case STK_CMD_READ_FUSE_PP:		gResult.data[0] = hv::ReadFuse(); len[0] = 3; break;
  case STK_CMD_PROGRAM_LOCK_PP:		gResult.status = hv::ppProgramLock(); break;
  case STK_CMD_READ_LOCK_PP:		gResult.data[0] = hv::ReadLock(); len[0] = 3; break;
  case STK_CMD_READ_SIGNATURE_PP:	gResult.data[0] = hv::ReadSignature(); len[0] = 3; break;
  case STK_CMD_READ_OSCCAL_PP:		gResult.data[0] = hv::ReadOsccal(); len[0] = 3; break;

  case STK_CMD_ENTER_PROGMODE_ISP:	gResult.status = isp::EnterProgmode(); break;
  case STK_CMD_LEAVE_PROGMODE_ISP:	isp::LeaveProgmode(); break;
  case STK_CMD_CHIP_ERASE_ISP:		gResult.status = isp::ChipErase(); break;
  case STK_CMD_PROGRAM_FLASH_ISP:	gResult.status = isp::ProgramMemory(false); break;
  case STK_CMD_READ_FLASH_ISP:		len = 1 + isp::ReadMemory(false); break;
  case STK_CMD_PROGRAM_EEPROM_ISP:	gResult.status = isp::ProgramMemory(true);
  case STK_CMD_READ_EEPROM_ISP:		len = 1 + isp::ReadMemory(true); break;
  case STK_CMD_PROGRAM_FUSE_ISP:	gResult.status = isp::ProgramFuse(); break;
  case STK_CMD_READ_FUSE_ISP:
  case STK_CMD_READ_LOCK_ISP:
  case STK_CMD_READ_SIGNATURE_ISP:
  case STK_CMD_READ_OSCCAL_ISP:		gResult.text[0] = isp::ReadFuse();
					gResult.text[1] = STK_STATUS_CMD_OK;
					len[0] = 4; break;
  case STK_CMD_PROGRAM_LOCK_ISP:	gResult.status = isp::ProgramFuse(); break;
  case STK_CMD_SPI_MULTI:		len = 1 + isp::Multi(); break;
  default:				gResult.status = STK_STATUS_CMD_FAILED;
 }
 SetTxMessage(len);
}

/* ------------------------------------------------------------------------- */

void SetRxChar(byte c) {
 if (timerLongTimeoutOccurred()) rxPos = 0;
 if (!rxPos) {     /* only accept STX as the first character */
  if (c == STK_STX) rxBuffer[rxPos++] = c;
 }else if (rxPos >= sizeof rxBuffer) rxPos = 0;  // overflow: reset state
 else{
  rxBuffer[rxPos++] = c;
  if (rxPos == 4) {     // do we have length word?
   rxLen = (rxBuffer[2]<<8 | c) + 6;
   if(rxLen > BUFFER_SIZE) rxPos = 0;    // illegal length: reset state
  }else if(rxPos == 5) {   /* check whether this is the token byte */
   if (c != STK_TOKEN) rxPos = 0;  /* reset state */
  }else if(rxPos > 4 && rxPos == rxLen) { /* message is complete */
   uchar sum = 0;
   uchar *p = rxBuffer;
   while(rxPos){ /* decrement rxPos down to 0 -> reset state */
    sum ^= *p++;
    rxPos--;
   }
   if (!sum) rxBlockAvailable = true;   /* check sum is correct, evaluate rx message */
   else{          /* checksum error */
    gResult.maincmd = STK_ANSWER_CKSUM_ERROR;
    gResult.status  = STK_ANSWER_CKSUM_ERROR;
    stk::SetTxMessage(2);
   }
  }
 }
 timerSetupLongTimeout(RX_TIMEOUT);
}

int GetTxCount() {
 return txLen - txPos;
}

int GetTxByte() {
 if (!txLen) return -1;
 byte c = txBuffer[txPos++];
 if (txPos >= txLen) txPos = txLen = 0;         /* transmit ready */
 return c;
}

void Poll() {
 if (!rxBlockAvailable) return;
 rxBlockAvailable = 0;
 EvaluateRxMessage();
}
}/*namespace*/
