/*
This module implements functions to control timing behavior.
It is based on a timer interrupt.
*/

#pragma once

#include <avr/io.h> /* for TCNT0 */
#include "utils.h"

extern volatile uchar   timerTimeoutCnt;
extern volatile uchar   timerLongTimeoutCnt;

/* ------------------------------------------------------------------------- */

#define TIMER_TICK_US       (64000000.0/(double)F_CPU)

void timerMsDelay(uchar ms);    /* ~1 ms precision, based on hardware timer */
void timerSetupTimeout(uchar msDuration);
void timerTicksDelay(uchar ticks);

#define TIMER_US_DELAY(us)  timerTicksDelay((uchar)((us)/TIMER_TICK_US))

/* ------------------------------------------------------------------------- */

static inline uchar   timerTimeoutOccurred(void)
{
    return timerTimeoutCnt == 0;
}

static inline void    timerSetupLongTimeout(uchar ms100Duration)
{
    timerLongTimeoutCnt = ms100Duration;
}

static inline uchar   timerLongTimeoutOccurred(void)
{
    return timerLongTimeoutCnt == 0;
}
